@@ -19,7 +19,7 @@
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.
 
-  Copyright (C) 2002-2004 Jan-ke Larsson
+  Copyright (C) 2002-2006 Jan-ke Larsson
 
 ************************************************************************/
 
@@ -106,10 +106,11 @@ dviunits SetChar(int32_t c)
   case FONT_TYPE_PK: DEBUG_PRINT(DEBUG_DVI,("\n  PK CHAR:\t")); break;
   case FONT_TYPE_T1: DEBUG_PRINT(DEBUG_DVI,("\n  T1 CHAR:\t")); break;
   case FONT_TYPE_FT: DEBUG_PRINT(DEBUG_DVI,("\n  FT CHAR:\t")); break;
+  default: DEBUG_PRINT(DEBUG_DVI,("\n  NO CHAR:\t"))
   }
   if (isprint(c))
     DEBUG_PRINT(DEBUG_DVI,("'%c' ",c));
-  DEBUG_PRINT(DEBUG_DVI,("%d at (%d,%d) tfmw %d", c,hh,vv,ptr->tfmw));
+  DEBUG_PRINT(DEBUG_DVI,("%d at (%d,%d) tfmw %d", c,hh,vv,ptr?ptr->tfmw:0));
 #endif
   if (currentfont->type==FONT_TYPE_VF) {
     return(SetVF(c));
@@ -198,7 +199,7 @@ void DrawCommand(unsigned char* command, void* parent /* dvi/vf */)
     break;
   case PUSH:
     if (sp >= STACK_SIZE)
-      Fatal("stack overflow");
+      Fatal("DVI stack overflow");
     stack[sp].h = h;
     stack[sp].v = v;
     stack[sp].w = w;
@@ -214,7 +215,7 @@ void DrawCommand(unsigned char* command, void* parent /* dvi/vf */)
   case POP:
     --sp;
     if (sp < 0)
-      Fatal("stack underflow");
+      Fatal("DVI stack underflow");
     h = stack[sp].h;
     v = stack[sp].v;
     w = stack[sp].w;
@@ -364,6 +365,9 @@ void DrawPages(void)
   if (dvi_pos!=NULL) {
     while(dvi_pos!=NULL) {
       SeekPage(dvi,dvi_pos);
+      Message(BE_NONQUIET,"[%d", dvi_pos->count[(flags & DVI_PAGENUM)?0:10]);
+      if (dvi_pos->count[(flags & DVI_PAGENUM)?0:10]!=dvi_pos->count[0])
+	Message(BE_NONQUIET," (%d)", dvi_pos->count[0]);
       x_max = y_max = INT32_MIN;
       x_min = y_min = INT32_MAX;
       DrawPage((dviunits)0,(dviunits)0);
@@ -406,9 +410,6 @@ void DrawPages(void)
 	DEBUG_PRINT(DEBUG_DVI,(" (%d)\n",dvi_pos->count[10]));
       }
 #endif
-      Message(BE_NONQUIET,"[%d", dvi_pos->count[(flags & DVI_PAGENUM)?0:10]);
-      if (dvi_pos->count[(flags & DVI_PAGENUM)?0:10]!=dvi_pos->count[0])
-	Message(BE_NONQUIET," (%d)", dvi_pos->count[0]);
       Message(REPORT_DEPTH," depth=%d", y_width-y_offset-1);
       Message(REPORT_HEIGHT," height=%d", y_offset+1);
       DrawPage(x_offset*dvi->conv*shrinkfactor,
@@ -420,11 +421,12 @@ void DrawPages(void)
 #endif
       } else {
 	exitcode=EXIT_FAILURE;
-	Message(BE_NONQUIET," not rendered");
+	Message(BE_NONQUIET,"(page not rendered)");
+	DestroyImage();
       }
       Message(BE_NONQUIET,"] ");
       fflush(stdout);
-      flags &= ~PAGE_GAVE_WARN;
+      flags &= ~(PAGE_GAVE_WARN | PAGE_TRUECOLOR);
       dvi_pos=NextPPage(dvi,dvi_pos);
     }
     Message(BE_NONQUIET,"\n");
