@@ -19,7 +19,7 @@
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.
 
-  Copyright (C) 2002-2004 Jan-ke Larsson
+  Copyright (C) 2002-2005 Jan-ke Larsson
 
 ************************************************************************/
 
@@ -48,10 +48,10 @@ int32_t myatodim(char ** p)
   while (i<8 && ((*p)[0]!=lengthnames[i][0] || (*p)[1]!=lengthnames[i][1]))
     i++; 
   if (i==8 && ((*p)[0]!=lengthnames[i][0] || (*p)[1]!=lengthnames[i][1]))
-    Warning("Unrecognized length unit \"%.2s\", assuming inches.",*p);
+    Warning("unrecognized length unit \"%.2s\", assuming inches",*p);
   while (**p != ',' && **p !='\0')
     (*p)++;
-  tmp *= lengthsp[i]*dpi/4736286L; /* convert sp to pixels */
+  tmp = tmp*lengthsp[i]*dpi/4736286L; /*   sp * dots/in / (sp/in), convert sp to pixels   */
   return((int32_t) tmp);
 }
 
